@@ -17,7 +17,6 @@ import jsonpatch
 from ironicclient import exceptions as ironic_exceptions
 
 from shade.exc import *  # noqa
-from shade import meta
 from shade import openstackcloud
 from shade import _tasks
 from shade import _utils
@@ -755,32 +754,30 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         # TODO(mordred) When this changes to REST, force interface=admin
         # in the adapter call
-        if self.cloud_config.get_api_version('identity').startswith('2'):
-            kwargs['service_type'] = type_ or service_type
+        if self._is_client_version('identity', 2):
+            url, key = '/OS-KSADM/services', 'OS-KSADM:service'
+            kwargs['type'] = type_ or service_type
         else:
+            url, key = '/services', 'service'
             kwargs['type'] = type_ or service_type
             kwargs['enabled'] = enabled
+        kwargs['name'] = name
 
-        with _utils.shade_exceptions(
-            "Failed to create service {name}".format(name=name)
-        ):
-            service = self.manager.submit_task(
-                _tasks.ServiceCreate(name=name, **kwargs)
-            )
-
+        msg = 'Failed to create service {name}'.format(name=name)
+        data = self._identity_client.post(
+            url, json={key: kwargs}, error_message=msg)
+        service = self._get_and_munchify(key, data)
         return _utils.normalize_keystone_services([service])[0]
 
     @_utils.valid_kwargs('name', 'enabled', 'type', 'service_type',
                          'description')
     def update_service(self, name_or_id, **kwargs):
         # NOTE(SamYaple): Service updates are only available on v3 api
-        if self.cloud_config.get_api_version('identity').startswith('2'):
+        if self._is_client_version('identity', 2):
             raise OpenStackCloudUnavailableFeature(
                 'Unavailable Feature: Service update requires Identity v3'
             )
 
-        # TODO(mordred) When this changes to REST, force interface=admin
-        # in the adapter call
         # NOTE(SamYaple): Keystone v3 only accepts 'type' but shade accepts
         #                 both 'type' and 'service_type' with a preference
         #                 towards 'type'
@@ -789,13 +786,17 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if type_ or service_type:
             kwargs['type'] = type_ or service_type
 
-        with _utils.shade_exceptions(
-            "Error in updating service {service}".format(service=name_or_id)
-        ):
-            service = self.manager.submit_task(
-                _tasks.ServiceUpdate(service=name_or_id, **kwargs)
-            )
+        if self._is_client_version('identity', 2):
+            url, key = '/OS-KSADM/services', 'OS-KSADM:service'
+        else:
+            url, key = '/services', 'service'
 
+        service = self.get_service(name_or_id)
+        msg = 'Error in updating service {service}'.format(service=name_or_id)
+        data = self._identity_client.patch(
+            '{url}/{id}'.format(url=url, id=service['id']), json={key: kwargs},
+            endpoint_filter={'interface': 'admin'}, error_message=msg)
+        service = self._get_and_munchify(key, data)
         return _utils.normalize_keystone_services([service])[0]
 
     def list_services(self):
@@ -806,10 +807,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: ``OpenStackCloudException`` if something goes wrong during the
             openstack API call.
         """
-        # TODO(mordred) When this changes to REST, force interface=admin
-        # in the adapter call
-        with _utils.shade_exceptions():
-            services = self.manager.submit_task(_tasks.ServiceList())
+        if self._is_client_version('identity', 2):
+            url, key = '/OS-KSADM/services', 'OS-KSADM:services'
+        else:
+            url, key = '/services', 'services'
+        data = self._identity_client.get(
+            url, endpoint_filter={'interface': 'admin'},
+            error_message="Failed to list services")
+        services = self._get_and_munchify(key, data)
         return _utils.normalize_keystone_services(services)
 
     def search_services(self, name_or_id=None, filters=None):
@@ -861,15 +866,15 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             self.log.debug("Service %s not found for deleting", name_or_id)
             return False
 
-        if self.cloud_config.get_api_version('identity').startswith('2'):
-            service_kwargs = {'id': service['id']}
+        if self._is_client_version('identity', 2):
+            url = '/OS-KSADM/services'
         else:
-            service_kwargs = {'service': service['id']}
-        # TODO(mordred) When this changes to REST, force interface=admin
-        # in the adapter call
-        with _utils.shade_exceptions("Failed to delete service {id}".format(
-                id=service['id'])):
-            self.manager.submit_task(_tasks.ServiceDelete(**service_kwargs))
+            url = '/services'
+
+        error_msg = 'Failed to delete service {id}'.format(id=service['id'])
+        self._identity_client.delete(
+            '{url}/{id}'.format(url=url, id=service['id']),
+            endpoint_filter={'interface': 'admin'}, error_message=error_msg)
 
         return True
 
@@ -914,7 +919,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         endpoint_args = []
         if url:
             urlkwargs = {}
-            if self.cloud_config.get_api_version('identity').startswith('2'):
+            if self._is_client_version('identity', 2):
                 if interface != 'public':
                     raise OpenStackCloudException(
                         "Error adding endpoint for service {service}."
@@ -929,24 +934,28 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 urlkwargs['interface'] = interface
             endpoint_args.append(urlkwargs)
         else:
-            expected_endpoints = {'public': public_url,
-                                  'internal': internal_url,
-                                  'admin': admin_url}
-            if self.cloud_config.get_api_version('identity').startswith('2'):
+            # NOTE(notmorgan): This is done as a list of tuples to ensure we
+            # have a deterministic order we try and create the endpoint
+            # elements. This is done mostly so that it is possible to test the
+            # requests themselves.
+            expected_endpoints = [('public', public_url),
+                                  ('internal', internal_url),
+                                  ('admin', admin_url)]
+            if self._is_client_version('identity', 2):
                 urlkwargs = {}
-                for interface, url in expected_endpoints.items():
+                for interface, url in expected_endpoints:
                     if url:
                         urlkwargs['{}url'.format(interface)] = url
                 endpoint_args.append(urlkwargs)
             else:
-                for interface, url in expected_endpoints.items():
+                for interface, url in expected_endpoints:
                     if url:
                         urlkwargs = {}
                         urlkwargs['url'] = url
                         urlkwargs['interface'] = interface
                         endpoint_args.append(urlkwargs)
 
-        if self.cloud_config.get_api_version('identity').startswith('2'):
+        if self._is_client_version('identity', 2):
             kwargs['service_id'] = service['id']
             # Keystone v2 requires 'region' arg even if it is None
             kwargs['region'] = region
@@ -975,7 +984,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                          'region')
     def update_endpoint(self, endpoint_id, **kwargs):
         # NOTE(SamYaple): Endpoint updates are only available on v3 api
-        if self.cloud_config.get_api_version('identity').startswith('2'):
+        if self._is_client_version('identity', 2):
             raise OpenStackCloudUnavailableFeature(
                 'Unavailable Feature: Endpoint update'
             )
@@ -1067,7 +1076,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         # TODO(mordred) When this changes to REST, force interface=admin
         # in the adapter call
-        if self.cloud_config.get_api_version('identity').startswith('2'):
+        if self._is_client_version('identity', 2):
             endpoint_kwargs = {'id': endpoint['id']}
         else:
             endpoint_kwargs = {'endpoint': endpoint['id']}
@@ -1077,24 +1086,24 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         return True
 
-    def create_domain(
-            self, name, description=None, enabled=True):
-        """Create a Keystone domain.
+    def create_domain(self, name, description=None, enabled=True):
+        """Create a domain.
 
         :param name: The name of the domain.
         :param description: A description of the domain.
         :param enabled: Is the domain enabled or not (default True).
 
-        :returns: a ``munch.Munch`` containing the domain description
+        :returns: a ``munch.Munch`` containing the domain representation.
 
-        :raise OpenStackCloudException: if the domain cannot be created
+        :raise OpenStackCloudException: if the domain cannot be created.
         """
-        with _utils.shade_exceptions("Failed to create domain {name}".format(
-                name=name)):
-            domain = self.manager.submit_task(_tasks.DomainCreate(
-                name=name,
-                description=description,
-                enabled=enabled))
+        domain_ref = {'name': name, 'enabled': enabled}
+        if description is not None:
+            domain_ref['description'] = description
+        msg = 'Failed to create domain {name}'.format(name=name)
+        data = self._identity_client.post(
+            '/domains', json={'domain': domain_ref}, error_message=msg)
+        domain = self._get_and_munchify('domain', data)
         return _utils.normalize_domains([domain])[0]
 
     def update_domain(
@@ -1112,15 +1121,20 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 )
             domain_id = dom['id']
 
-        with _utils.shade_exceptions(
-                "Error in updating domain {domain}".format(domain=domain_id)):
-            domain = self.manager.submit_task(_tasks.DomainUpdate(
-                domain=domain_id, name=name, description=description,
-                enabled=enabled))
+        domain_ref = {}
+        domain_ref.update({'name': name} if name else {})
+        domain_ref.update({'description': description} if description else {})
+        domain_ref.update({'enabled': enabled} if enabled is not None else {})
+
+        error_msg = "Error in updating domain {id}".format(id=domain_id)
+        data = self._identity_client.patch(
+            '/domains/{id}'.format(id=domain_id),
+            json={'domain': domain_ref}, error_message=error_msg)
+        domain = self._get_and_munchify('domain', data)
         return _utils.normalize_domains([domain])[0]
 
     def delete_domain(self, domain_id=None, name_or_id=None):
-        """Delete a Keystone domain.
+        """Delete a domain.
 
         :param domain_id: ID of the domain to delete.
         :param name_or_id: Name or ID of the domain to delete.
@@ -1135,23 +1149,22 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 raise OpenStackCloudException(
                     "You must pass either domain_id or name_or_id value"
                 )
-            dom = self.get_domain(None, name_or_id)
+            dom = self.get_domain(name_or_id=name_or_id)
             if dom is None:
                 self.log.debug(
                     "Domain %s not found for deleting", name_or_id)
                 return False
             domain_id = dom['id']
 
-        with _utils.shade_exceptions(
-                "Failed to delete domain {id}".format(id=domain_id)):
-            # Deleting a domain is expensive, so disabling it first increases
-            # the changes of success
-            domain = self.update_domain(domain_id, enabled=False)
-            self.manager.submit_task(_tasks.DomainDelete(domain=domain['id']))
+        # A domain must be disabled before deleting
+        self.update_domain(domain_id, enabled=False)
+        error_msg = "Failed to delete domain {id}".format(id=domain_id)
+        self._identity_client.delete('/domains/{id}'.format(id=domain_id),
+                                     error_message=error_msg)
 
         return True
 
-    def list_domains(self):
+    def list_domains(self, **filters):
         """List Keystone domains.
 
         :returns: a list of ``munch.Munch`` containing the domain description.
@@ -1159,8 +1172,9 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        with _utils.shade_exceptions("Failed to list domains"):
-            domains = self.manager.submit_task(_tasks.DomainList())
+        data = self._identity_client.get(
+            '/domains', params=filters, error_message="Failed to list domains")
+        domains = self._get_and_munchify('domains', data)
         return _utils.normalize_domains(domains)
 
     def search_domains(self, filters=None, name_or_id=None):
@@ -1185,10 +1199,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             domains = self.list_domains()
             return _utils._filter_list(domains, name_or_id, filters)
         else:
-            with _utils.shade_exceptions("Failed to list domains"):
-                domains = self.manager.submit_task(
-                    _tasks.DomainList(**filters))
-            return _utils.normalize_domains(domains)
+            return self.list_domains(**filters)
 
     def get_domain(self, domain_id=None, name_or_id=None, filters=None):
         """Get exactly one Keystone domain.
@@ -1209,55 +1220,69 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             the openstack API call.
         """
         if domain_id is None:
+            # NOTE(SamYaple): search_domains() has filters and name_or_id
+            # in the wrong positional order which prevents _get_entity from
+            # being able to return quickly if passing a domain object so we
+            # duplicate that logic here
+            if hasattr(name_or_id, 'id'):
+                return name_or_id
             return _utils._get_entity(self.search_domains, filters, name_or_id)
         else:
-            with _utils.shade_exceptions(
-                "Failed to get domain "
-                "{domain_id}".format(domain_id=domain_id)
-            ):
-                domain = self.manager.submit_task(
-                    _tasks.DomainGet(domain=domain_id))
+            error_msg = 'Failed to get domain {id}'.format(id=domain_id)
+            data = self._identity_client.get(
+                '/domains/{id}'.format(id=domain_id),
+                error_message=error_msg)
+            domain = self._get_and_munchify('domain', data)
             return _utils.normalize_domains([domain])[0]
 
+    @_utils.valid_kwargs('domain_id')
     @_utils.cache_on_arguments()
-    def list_groups(self):
+    def list_groups(self, **kwargs):
         """List Keystone Groups.
 
+        :param domain_id: domain id.
+
         :returns: A list of ``munch.Munch`` containing the group description.
 
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        with _utils.shade_exceptions("Failed to list groups"):
-            groups = self.manager.submit_task(_tasks.GroupList())
-        return _utils.normalize_groups(groups)
+        data = self._identity_client.get(
+            '/groups', params=kwargs, error_message="Failed to list groups")
+        return _utils.normalize_groups(self._get_and_munchify('groups', data))
 
-    def search_groups(self, name_or_id=None, filters=None):
+    @_utils.valid_kwargs('domain_id')
+    def search_groups(self, name_or_id=None, filters=None, **kwargs):
         """Search Keystone groups.
 
         :param name: Group name or id.
         :param filters: A dict containing additional filters to use.
+        :param domain_id: domain id.
 
         :returns: A list of ``munch.Munch`` containing the group description.
 
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        groups = self.list_groups()
-        return _utils._filter_list(groups, name_or_id, filters)
+        groups = self.list_groups(**kwargs)
+        return _utils._filter_list(groups, name_or_id, filters,
+                                   **kwargs)
 
-    def get_group(self, name_or_id, filters=None):
+    @_utils.valid_kwargs('domain_id')
+    def get_group(self, name_or_id, filters=None, **kwargs):
         """Get exactly one Keystone group.
 
         :param id: Group name or id.
         :param filters: A dict containing additional filters to use.
+        :param domain_id: domain id.
 
         :returns: A ``munch.Munch`` containing the group description.
 
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        return _utils._get_entity(self.search_groups, name_or_id, filters)
+        return _utils._get_entity(self.search_groups, name_or_id, filters,
+                                  **kwargs)
 
     def create_group(self, name, description, domain=None):
         """Create a group.
@@ -1271,30 +1296,33 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        with _utils.shade_exceptions(
-            "Error creating group {group}".format(group=name)
-        ):
-            domain_id = None
-            if domain:
-                dom = self.get_domain(domain)
-                if not dom:
-                    raise OpenStackCloudException(
-                        "Creating group {group} failed: Invalid domain "
-                        "{domain}".format(group=name, domain=domain)
-                    )
-                domain_id = dom['id']
+        group_ref = {'name': name}
+        if description:
+            group_ref['description'] = description
+        if domain:
+            dom = self.get_domain(domain)
+            if not dom:
+                raise OpenStackCloudException(
+                    "Creating group {group} failed: Invalid domain "
+                    "{domain}".format(group=name, domain=domain)
+                )
+            group_ref['domain_id'] = dom['id']
 
-            group = self.manager.submit_task(_tasks.GroupCreate(
-                name=name, description=description, domain=domain_id)
-            )
+        error_msg = "Error creating group {group}".format(group=name)
+        data = self._identity_client.post(
+            '/groups', json={'group': group_ref}, error_message=error_msg)
+        group = self._get_and_munchify('group', data)
         self.list_groups.invalidate(self)
         return _utils.normalize_groups([group])[0]
 
-    def update_group(self, name_or_id, name=None, description=None):
+    @_utils.valid_kwargs('domain_id')
+    def update_group(self, name_or_id, name=None, description=None,
+                     **kwargs):
         """Update an existing group
 
         :param string name: New group name.
         :param string description: New group description.
+        :param domain_id: domain id.
 
         :returns: A ``munch.Munch`` containing the group description.
 
@@ -1302,41 +1330,47 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             the openstack API call.
         """
         self.list_groups.invalidate(self)
-        group = self.get_group(name_or_id)
+        group = self.get_group(name_or_id, **kwargs)
         if group is None:
             raise OpenStackCloudException(
                 "Group {0} not found for updating".format(name_or_id)
             )
 
-        with _utils.shade_exceptions(
-            "Unable to update group {name}".format(name=name_or_id)
-        ):
-            group = self.manager.submit_task(_tasks.GroupUpdate(
-                group=group['id'], name=name, description=description))
+        group_ref = {}
+        if name:
+            group_ref['name'] = name
+        if description:
+            group_ref['description'] = description
 
+        error_msg = "Unable to update group {name}".format(name=name_or_id)
+        data = self._identity_client.patch(
+            '/groups/{id}'.format(id=group['id']),
+            json={'group': group_ref}, error_message=error_msg)
+        group = self._get_and_munchify('group', data)
         self.list_groups.invalidate(self)
         return _utils.normalize_groups([group])[0]
 
-    def delete_group(self, name_or_id):
+    @_utils.valid_kwargs('domain_id')
+    def delete_group(self, name_or_id, **kwargs):
         """Delete a group
 
         :param name_or_id: ID or name of the group to delete.
+        :param domain_id: domain id.
 
         :returns: True if delete succeeded, False otherwise.
 
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        group = self.get_group(name_or_id)
+        group = self.get_group(name_or_id, **kwargs)
         if group is None:
             self.log.debug(
                 "Group %s not found for deleting", name_or_id)
             return False
 
-        with _utils.shade_exceptions(
-            "Unable to delete group {name}".format(name=name_or_id)
-        ):
-            self.manager.submit_task(_tasks.GroupDelete(group=group['id']))
+        error_msg = "Unable to delete group {name}".format(name=name_or_id)
+        self._identity_client.delete('/groups/{id}'.format(id=group['id']),
+                                     error_message=error_msg)
 
         self.list_groups.invalidate(self)
         return True
@@ -1349,10 +1383,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: ``OpenStackCloudException``: if something goes wrong during
             the openstack API call.
         """
-        with _utils.shade_exceptions():
-            roles = self.manager.submit_task(_tasks.RoleList())
-
-        return roles
+        v2 = self._is_client_version('identity', 2)
+        url = '/OS-KSADM/roles' if v2 else '/roles'
+        data = self._identity_client.get(
+            url, error_message="Failed to list roles")
+        return _utils.normalize_roles(self._get_and_munchify('roles', data))
 
     def search_roles(self, name_or_id=None, filters=None):
         """Seach Keystone roles.
@@ -1450,7 +1485,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not filters:
             filters = {}
 
-        if self.cloud_config.get_api_version('identity').startswith('2'):
+        if self._is_client_version('identity', 2):
             if filters.get('project') is None or filters.get('user') is None:
                 raise OpenStackCloudException(
                     "Must provide project and user for keystone v2"
@@ -1496,11 +1531,12 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             }
             if flavorid == 'auto':
                 payload['id'] = None
-            flavor = self._compute_client.post(
+            data = self._compute_client.post(
                 '/flavors',
                 json=dict(flavor=payload))
 
-        return self._normalize_flavor(flavor)
+        return self._normalize_flavor(
+            self._get_and_munchify('flavor', data))
 
     def delete_flavor(self, name_or_id):
         """Delete a flavor
@@ -1605,9 +1641,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         with _utils.shade_exceptions("Error trying to list access from "
                                      "flavor ID {flavor}".format(
                 flavor=flavor_id)):
-            projects = self._compute_client.get(
+            data = self._compute_client.get(
                 '/flavors/{id}/os-flavor-access'.format(id=flavor_id))
-        return _utils.normalize_flavor_accesses(projects)
+        return _utils.normalize_flavor_accesses(
+            self._get_and_munchify('flavor_access', data))
 
     def create_role(self, name):
         """Create a Keystone role.
@@ -1618,11 +1655,13 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         :raise OpenStackCloudException: if the role cannot be created
         """
-        with _utils.shade_exceptions():
-            role = self.manager.submit_task(
-                _tasks.RoleCreate(name=name)
-            )
-        return role
+        v2 = self._is_client_version('identity', 2)
+        url = '/OS-KSADM/roles' if v2 else '/roles'
+        msg = 'Failed to create role {name}'.format(name=name)
+        data = self._identity_client.post(
+            url, json={'role': {'name': name}}, error_message=msg)
+        role = self._get_and_munchify('role', data)
+        return _utils.normalize_roles([role])[0]
 
     def delete_role(self, name_or_id):
         """Delete a Keystone role.
@@ -1640,9 +1679,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 "Role %s not found for deleting", name_or_id)
             return False
 
-        with _utils.shade_exceptions("Unable to delete role {name}".format(
-                name=name_or_id)):
-            self.manager.submit_task(_tasks.RoleDelete(role=role['id']))
+        v2 = self._is_client_version('identity', 2)
+        url = '{preffix}/{id}'.format(
+            preffix='/OS-KSADM/roles' if v2 else '/roles', id=role['id'])
+        error_msg = "Unable to delete role {name}".format(name=name_or_id)
+        self._identity_client.delete(url, error_message=error_msg)
 
         return True
 
@@ -1654,8 +1695,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         data = {'role': role.id}
 
         # domain and group not available in keystone v2.0
-        keystone_version = self.cloud_config.get_api_version('identity')
-        is_keystone_v2 = keystone_version.startswith('2')
+        is_keystone_v2 = self._is_client_version('identity', 2)
 
         filters = {}
         if not is_keystone_v2 and domain:
@@ -1714,7 +1754,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if data.get('user') is None and data.get('group') is None:
             raise OpenStackCloudException(
                 'Must specify either a user or a group')
-        if self.cloud_config.get_api_version('identity').startswith('2') and \
+        if self._is_client_version('identity', 2) and \
                 data.get('project') is None:
             raise OpenStackCloudException(
                 'Must specify project for keystone v2')
@@ -1726,7 +1766,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         with _utils.shade_exceptions(
                 "Error granting access to role: {0}".format(
                 data)):
-            if self.cloud_config.get_api_version('identity').startswith('2'):
+            if self._is_client_version('identity', 2):
                 data['tenant'] = data.pop('project')
                 self.manager.submit_task(_tasks.RoleAddUser(**data))
             else:
@@ -1777,7 +1817,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if data.get('user') is None and data.get('group') is None:
             raise OpenStackCloudException(
                 'Must specify either a user or a group')
-        if self.cloud_config.get_api_version('identity').startswith('2') and \
+        if self._is_client_version('identity', 2) and \
                 data.get('project') is None:
             raise OpenStackCloudException(
                 'Must specify project for keystone v2')
@@ -1789,7 +1829,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         with _utils.shade_exceptions(
                 "Error revoking access to role: {0}".format(
                 data)):
-            if self.cloud_config.get_api_version('identity').startswith('2'):
+            if self._is_client_version('identity', 2):
                 data['tenant'] = data.pop('project')
                 self.manager.submit_task(_tasks.RoleRemoveUser(**data))
             else:
@@ -1811,9 +1851,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :returns: A list of hypervisor ``munch.Munch``.
         """
 
-        return self._compute_client.get(
+        data = self._compute_client.get(
             '/os-hypervisors/detail',
             error_message="Error fetching hypervisor list")
+        return self._get_and_munchify('hypervisors', data)
 
     def search_aggregates(self, name_or_id=None, filters=None):
         """Seach host aggregates.
@@ -1835,9 +1876,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :returns: A list of aggregate dicts.
 
         """
-        return self._compute_client.get(
+        data = self._compute_client.get(
             '/os-aggregates',
             error_message="Error fetching aggregate list")
+        return self._get_and_munchify('aggregates', data)
 
     def get_aggregate(self, name_or_id, filters=None):
         """Get an aggregate by name or ID.
@@ -1870,7 +1912,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         :raises: OpenStackCloudException on operation error.
         """
-        return self._compute_client.post(
+        data = self._compute_client.post(
             '/os-aggregates',
             json={'aggregate': {
                 'name': name,
@@ -1878,6 +1920,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             }},
             error_message="Unable to create host aggregate {name}".format(
                 name=name))
+        return self._get_and_munchify('aggregate', data)
 
     @_utils.valid_kwargs('name', 'availability_zone')
     def update_aggregate(self, name_or_id, **kwargs):
@@ -1896,11 +1939,12 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             raise OpenStackCloudException(
                 "Host aggregate %s not found." % name_or_id)
 
-        return self._compute_client.put(
+        data = self._compute_client.put(
             '/os-aggregates/{id}'.format(id=aggregate['id']),
             json={'aggregate': kwargs},
             error_message="Error updating aggregate {name}".format(
                 name=name_or_id))
+        return self._get_and_munchify('aggregate', data)
 
     def delete_aggregate(self, name_or_id):
         """Delete a host aggregate.
@@ -1942,10 +1986,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         err_msg = "Unable to set metadata for host aggregate {name}".format(
             name=name_or_id)
 
-        return self._compute_client.post(
+        data = self._compute_client.post(
             '/os-aggregates/{id}/action'.format(id=aggregate['id']),
             json={'set_metadata': {'metadata': metadata}},
             error_message=err_msg)
+        return self._get_and_munchify('aggregate', data)
 
     def add_host_to_aggregate(self, name_or_id, host_name):
         """Add a host to an aggregate.
@@ -2006,7 +2051,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             error_message="Unable to get volume type access"
                           " {name}".format(name=name_or_id))
         return self._normalize_volume_type_accesses(
-            meta.get_and_munchify('volume_type_access', data))
+            self._get_and_munchify('volume_type_access', data))
 
     def add_volume_type_access(self, name_or_id, project_id):
         """Grant access on a volume_type to a project.
@@ -2091,8 +2136,9 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        return self._compute_client.get(
+        data = self._compute_client.get(
             '/os-quota-sets/{project}'.format(project=proj.id))
+        return self._get_and_munchify('quota_set', data)
 
     def delete_compute_quotas(self, name_or_id):
         """ Delete quota for a project
@@ -2164,13 +2210,13 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             raise OpenStackCloudException("project does not exist: {}".format(
                 name=proj.id))
 
-        usage = self._compute_client.get(
+        data = self._compute_client.get(
             '/os-simple-tenant-usage/{project}'.format(project=proj.id),
             params=dict(start=start.isoformat(), end=end.isoformat()),
             error_message="Unable to get usage for project: {name}".format(
                 name=proj.id))
-
-        return self._normalize_compute_usage(usage)
+        return self._normalize_compute_usage(
+            self._get_and_munchify('tenant_usage', data))
 
     def set_volume_quotas(self, name_or_id, **kwargs):
         """ Set a volume quota in a project
@@ -2204,9 +2250,10 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         if not proj:
             raise OpenStackCloudException("project does not exist")
 
-        return self._volume_client.get(
+        data = self._volume_client.get(
             '/os-quota-sets/{tenant_id}'.format(tenant_id=proj.id),
             error_message="cinder client call failed")
+        return self._get_and_munchify('quota_set', data)
 
     def delete_volume_quotas(self, name_or_id):
         """ Delete volume quotas for a project
@@ -2256,10 +2303,11 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         proj = self.get_project(name_or_id)
         if not proj:
             raise OpenStackCloudException("project does not exist")
-        return self._network_client.get(
+        data = self._network_client.get(
             '/quotas/{project_id}.json'.format(project_id=proj.id),
             error_message=("Error fetching Neutron's quota for "
                            "project {0}".format(proj.id)))
+        return self._get_and_munchify('quota', data)
 
     def delete_network_quotas(self, name_or_id):
         """ Delete network quotas for a project
@@ -2285,5 +2333,6 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: OpenStackCloudException on operation error.
         """
         with _utils.shade_exceptions("Error fetching Magnum services list"):
+            data = self._container_infra_client.get('/mservices')
             return self._normalize_magnum_services(
-                self._container_infra_client.get('/mservices'))
+                self._get_and_munchify('mservices', data))
