@@ -17,6 +17,7 @@ test_compute
 Functional tests for `shade` compute methods.
 """
 
+from fixtures import TimeoutException
 import six
 
 from shade import exc
@@ -51,10 +52,19 @@ class TestCompute(base.BaseFunctionalTestCase):
         if not server:
             return
         volumes = self.user_cloud.get_volumes(server)
-        self.user_cloud.delete_server(server.name, wait=True)
-        for volume in volumes:
-            if volume.status != 'deleting':
-                self.user_cloud.delete_volume(volume.id, wait=True)
+        try:
+            self.user_cloud.delete_server(server.name, wait=True)
+            for volume in volumes:
+                if volume.status != 'deleting':
+                    self.user_cloud.delete_volume(volume.id, wait=True)
+        except (exc.OpenStackCloudTimeout, TimeoutException):
+            # Ups, some timeout occured during process of deletion server
+            # or volumes, so now we will try to call delete each of them
+            # once again and we will try to live with it
+            self.user_cloud.delete_server(server.name)
+            for volume in volumes:
+                self.operator_cloud.delete_volume(
+                    volume.id, wait=False, force=True)
 
     def test_create_and_delete_server(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -112,7 +122,7 @@ class TestCompute(base.BaseFunctionalTestCase):
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
         self.assertEqual(self.flavor.id, server['flavor']['id'])
-        self.assertEqual(True, server['has_config_drive'])
+        self.assertTrue(server['has_config_drive'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(self.server_name, wait=True))
@@ -132,7 +142,7 @@ class TestCompute(base.BaseFunctionalTestCase):
         self.assertEqual(self.server_name, server['name'])
         self.assertEqual(self.image.id, server['image']['id'])
         self.assertEqual(self.flavor.id, server['flavor']['id'])
-        self.assertEqual(False, server['has_config_drive'])
+        self.assertFalse(server['has_config_drive'])
         self.assertIsNotNone(server['adminPass'])
         self.assertTrue(
             self.user_cloud.delete_server(
@@ -183,12 +193,11 @@ class TestCompute(base.BaseFunctionalTestCase):
             image=self.image,
             flavor=self.flavor,
             wait=True)
-        for _ in _utils._iterate_timeout(
-                5, "Did not get more than 0 lines in the console log"):
-            log = self.user_cloud.get_server_console(server=server)
-            self.assertTrue(isinstance(log, six.string_types))
-            if len(log) > 0:
-                break
+        # _get_server_console_output does not trap HTTP exceptions, so this
+        # returning a string tests that the call is correct. Testing that
+        # the cloud returns actual data in the output is out of scope.
+        log = self.user_cloud._get_server_console_output(server_id=server.id)
+        self.assertTrue(isinstance(log, six.string_types))
 
     def test_get_server_console_name_or_id(self):
         self.addCleanup(self._cleanup_servers_and_volumes, self.server_name)
@@ -197,12 +206,8 @@ class TestCompute(base.BaseFunctionalTestCase):
             image=self.image,
             flavor=self.flavor,
             wait=True)
-        for _ in _utils._iterate_timeout(
-                5, "Did not get more than 0 lines in the console log"):
-            log = self.user_cloud.get_server_console(server=self.server_name)
-            self.assertTrue(isinstance(log, six.string_types))
-            if len(log) > 0:
-                break
+        log = self.user_cloud.get_server_console(server=self.server_name)
+        self.assertTrue(isinstance(log, six.string_types))
 
     def test_list_availability_zone_names(self):
         self.assertEqual(
@@ -272,7 +277,7 @@ class TestCompute(base.BaseFunctionalTestCase):
         volume = self.user_cloud.get_volume(volume_id)
         self.assertIsNotNone(volume)
         self.assertEqual(volume['name'], volume['display_name'])
-        self.assertEqual(True, volume['bootable'])
+        self.assertTrue(volume['bootable'])
         self.assertEqual(server['id'], volume['attachments'][0]['server_id'])
         self.assertTrue(self.user_cloud.delete_server(server.id, wait=True))
         self._wait_for_detach(volume.id)
@@ -336,7 +341,7 @@ class TestCompute(base.BaseFunctionalTestCase):
         volume = self.user_cloud.get_volume(volume_id)
         self.assertIsNotNone(volume)
         self.assertEqual(volume['name'], volume['display_name'])
-        self.assertEqual(True, volume['bootable'])
+        self.assertTrue(volume['bootable'])
         self.assertEqual([], volume['attachments'])
         self._wait_for_detach(volume.id)
         self.assertTrue(self.user_cloud.delete_volume(volume_id))
