@@ -25,7 +25,7 @@ from shade.exc import OpenStackCloudException
 from shade.tests.functional import base
 
 
-class TestProject(base.BaseFunctionalTestCase):
+class TestProject(base.KeystoneBaseFunctionalTestCase):
 
     def setUp(self):
         super(TestProject, self).setUp()
@@ -67,18 +67,32 @@ class TestProject(base.BaseFunctionalTestCase):
         params = {
             'name': project_name,
             'description': 'test_update_project',
+            'enabled': True
         }
         if self.identity_version == '3':
             params['domain_id'] = \
                 self.operator_cloud.get_domain('default')['id']
 
         project = self.operator_cloud.create_project(**params)
-        updated_project = self.operator_cloud.update_project(project_name,
-                                                             description='new')
+        updated_project = self.operator_cloud.update_project(
+            project_name, enabled=False, description='new')
         self.assertIsNotNone(updated_project)
         self.assertEqual(project['id'], updated_project['id'])
         self.assertEqual(project['name'], updated_project['name'])
         self.assertEqual(updated_project['description'], 'new')
+        self.assertTrue(project['enabled'])
+        self.assertFalse(updated_project['enabled'])
+
+        # Revert the description and verify the project is still disabled
+        updated_project = self.operator_cloud.update_project(
+            project_name, description=params['description'])
+        self.assertIsNotNone(updated_project)
+        self.assertEqual(project['id'], updated_project['id'])
+        self.assertEqual(project['name'], updated_project['name'])
+        self.assertEqual(project['description'],
+                         updated_project['description'])
+        self.assertTrue(project['enabled'])
+        self.assertFalse(updated_project['enabled'])
 
     def test_delete_project(self):
         project_name = self.new_project_name + '_delete'
