@@ -17,7 +17,6 @@ import collections
 import time
 import uuid
 
-from distutils import version as du_version
 import fixtures
 import mock
 import os
@@ -260,9 +259,11 @@ class RequestsMockTestCase(BaseTestCase):
             response['enabled'] = enabled
             request['enabled'] = enabled
         response.setdefault('enabled', True)
+        request.setdefault('enabled', True)
         if description:
             response['description'] = description
             request['description'] = description
+        request.setdefault('description', None)
         if v3:
             project_key = 'project'
         else:
@@ -276,7 +277,7 @@ class RequestsMockTestCase(BaseTestCase):
         name = name or self.getUniqueString('groupname')
         domain_id = uuid.UUID(domain_id or uuid.uuid4().hex).hex
         response = {'id': group_id, 'name': name, 'domain_id': domain_id}
-        request = {'name': name}
+        request = {'name': name, 'domain_id': domain_id}
         if description is not None:
             response['description'] = description
             request['description'] = description
@@ -291,11 +292,12 @@ class RequestsMockTestCase(BaseTestCase):
         user_id = uuid.uuid4().hex
 
         response = {'name': name, 'id': user_id}
-        request = {'name': name, 'password': password, 'tenantId': None}
+        request = {'name': name, 'password': password}
 
         if kwargs.get('domain_id'):
             kwargs['domain_id'] = uuid.UUID(kwargs['domain_id']).hex
             response['domain_id'] = kwargs.pop('domain_id')
+            request['domain_id'] = response['domain_id']
 
         response['email'] = kwargs.pop('email', None)
         request['email'] = response['email']
@@ -374,6 +376,8 @@ class RequestsMockTestCase(BaseTestCase):
         response = {'id': endpoint_id, 'service_id': service_id,
                     'region': region}
         v3_endpoints = {}
+        request = response.copy()
+        request.pop('id')
         if admin_url:
             response['adminURL'] = admin_url
             v3_endpoints['admin'] = self._get_endpoint_v3_data(
@@ -388,6 +392,8 @@ class RequestsMockTestCase(BaseTestCase):
                 service_id, region, public_url, interface='public')
         request = response.copy()
         request.pop('id')
+        for u in ('publicURL', 'internalURL', 'adminURL'):
+            request[u.lower()] = request.pop(u, None)
         return _EndpointDataV2(endpoint_id, service_id, region, public_url,
                                internal_url, admin_url, v3_endpoints,
                                {'endpoint': response}, {'endpoint': request})
@@ -414,7 +420,7 @@ class RequestsMockTestCase(BaseTestCase):
                      'X-Subject-Token': self.getUniqueString('KeystoneToken')},
                  text=open(os.path.join(
                      self.fixtures_directory, catalog), 'r').read()
-                 )
+                 ),
         ])
         self._make_test_cloud(identity_api_version='3')
 
@@ -423,15 +429,6 @@ class RequestsMockTestCase(BaseTestCase):
         self.calls = []
         self._uri_registry.clear()
 
-        # occ > 1.26.0 fixes keystoneclient construction. Unfortunately, it
-        # breaks our mocking of what keystoneclient does here. Since we're
-        # close to just getting rid of ksc anyway, just put in a version match
-        occ_version = du_version.StrictVersion(occ.__version__)
-        if occ_version > du_version.StrictVersion('1.26.0'):
-            endpoint_uri = 'https://identity.example.com/v2.0'
-        else:
-            endpoint_uri = 'https://identity.example.com/'
-
         self.__do_register_uris([
             dict(method='GET', uri='https://identity.example.com/',
                  text=open(self.discovery_json, 'r').read()),
@@ -439,25 +436,11 @@ class RequestsMockTestCase(BaseTestCase):
                  text=open(os.path.join(
                      self.fixtures_directory, 'catalog-v2.json'), 'r').read()
                  ),
-            dict(method='GET', uri=endpoint_uri,
-                 text=open(self.discovery_json, 'r').read()),
-            dict(method='GET', uri='https://identity.example.com/',
-                 text=open(self.discovery_json, 'r').read())
         ])
 
         self._make_test_cloud(cloud_name='_test_cloud_v2_',
                               identity_api_version='2.0')
 
-    def _add_discovery_uri_call(self):
-        # NOTE(notmorgan): Temp workaround for transition to requests
-        # mock for cases keystoneclient is still mocked directly. This allows
-        # us to inject another call to discovery where needed in a test that
-        # no longer mocks out kyestoneclient and performs the extra round
-        # trips.
-        self.__do_register_uris([
-            dict(method='GET', uri='https://identity.example.com/',
-                 text=open(self.discovery_json, 'r').read())])
-
     def _make_test_cloud(self, cloud_name='_test_cloud_', **kwargs):
         test_cloud = os.environ.get('SHADE_OS_CLOUD', cloud_name)
         self.cloud_config = self.config.get_one_cloud(
@@ -552,6 +535,12 @@ class RequestsMockTestCase(BaseTestCase):
             key = '{method}|{uri}|{params}'.format(
                 method=method, uri=uri, params=kw_params)
             validate = to_mock.pop('validate', {})
+            valid_keys = set(['json', 'headers', 'params'])
+            invalid_keys = set(validate.keys()) - valid_keys
+            if invalid_keys:
+                raise TypeError(
+                    "Invalid values passed to validate: {keys}".format(
+                        keys=invalid_keys))
             headers = structures.CaseInsensitiveDict(to_mock.pop('headers',
                                                                  {}))
             if 'content-type' not in headers:
