@@ -44,7 +44,9 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
             service_data.service_id, public_url=self._dummy_url(),
             internal_url=self._dummy_url(), admin_url=self._dummy_url())
         other_endpoint_data = self._get_endpoint_v2_data(
-            service_data.service_id, public_url=self._dummy_url())
+            service_data.service_id, region=endpoint_data.region,
+            public_url=endpoint_data.public_url)
+        # correct the keys
 
         self.register_uris([
             dict(method='GET',
@@ -57,7 +59,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(base_url_append=None),
                  status_code=200,
                  json=endpoint_data.json_response,
-                 validate=endpoint_data.json_request),
+                 validate=dict(json=endpoint_data.json_request)),
             dict(method='GET',
                  uri=self.get_mock_url(
                      resource='services', base_url_append='OS-KSADM'),
@@ -77,7 +79,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(base_url_append=None),
                  status_code=200,
                  json=other_endpoint_data.json_response,
-                 validate=other_endpoint_data.json_request)
+                 validate=dict(json=other_endpoint_data.json_request))
         ])
 
         endpoints = self.op_cloud.create_endpoint(
@@ -127,7 +129,6 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
         self.assert_calls()
 
     def test_create_endpoint_v3(self):
-        self._add_discovery_uri_call()
         service_data = self._get_service_data()
         public_endpoint_data = self._get_endpoint_v3_data(
             service_id=service_data.service_id, interface='public',
@@ -152,7 +153,8 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(),
                  status_code=200,
                  json=public_endpoint_data_disabled.json_response,
-                 validate=public_endpoint_data_disabled.json_request),
+                 validate=dict(
+                     json=public_endpoint_data_disabled.json_request)),
             dict(method='GET',
                  uri=self.get_mock_url(
                      append=[public_endpoint_data_disabled.endpoint_id]),
@@ -167,7 +169,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(),
                  status_code=200,
                  json=public_endpoint_data.json_response,
-                 validate=public_endpoint_data.json_request),
+                 validate=dict(json=public_endpoint_data.json_request)),
             dict(method='GET',
                  uri=self.get_mock_url(
                      append=[public_endpoint_data.endpoint_id]),
@@ -177,7 +179,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(),
                  status_code=200,
                  json=internal_endpoint_data.json_response,
-                 validate=internal_endpoint_data.json_request),
+                 validate=dict(json=internal_endpoint_data.json_request)),
             dict(method='GET',
                  uri=self.get_mock_url(
                      append=[internal_endpoint_data.endpoint_id]),
@@ -187,7 +189,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                  uri=self.get_mock_url(),
                  status_code=200,
                  json=admin_endpoint_data.json_response,
-                 validate=admin_endpoint_data.json_request),
+                 validate=dict(json=admin_endpoint_data.json_request)),
             dict(method='GET',
                  uri=self.get_mock_url(
                      append=[admin_endpoint_data.endpoint_id]),
@@ -251,17 +253,19 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
                           self.op_cloud.update_endpoint, 'endpoint_id')
 
     def test_update_endpoint_v3(self):
-        self._add_discovery_uri_call()
         service_data = self._get_service_data()
+        dummy_url = self._dummy_url()
         endpoint_data = self._get_endpoint_v3_data(
             service_id=service_data.service_id, interface='admin',
             enabled=False)
+        reference_request = endpoint_data.json_request.copy()
+        reference_request['endpoint']['url'] = dummy_url
         self.register_uris([
             dict(method='PATCH',
                  uri=self.get_mock_url(append=[endpoint_data.endpoint_id]),
                  status_code=200,
                  json=endpoint_data.json_response,
-                 validate=endpoint_data.json_request),
+                 validate=dict(json=reference_request)),
             dict(method='GET',
                  uri=self.get_mock_url(append=[endpoint_data.endpoint_id]),
                  status_code=200,
@@ -271,7 +275,7 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
             endpoint_data.endpoint_id,
             service_name_or_id=service_data.service_id,
             region=endpoint_data.region,
-            url=self._dummy_url(),
+            url=dummy_url,
             interface=endpoint_data.interface,
             enabled=False
         )
@@ -289,7 +293,6 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
         self.assert_calls()
 
     def test_list_endpoints(self):
-        self._add_discovery_uri_call()
         endpoints_data = [self._get_endpoint_v3_data() for e in range(1, 10)]
         self.register_uris([
             dict(method='GET',
@@ -317,7 +320,6 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
         self.assert_calls()
 
     def test_search_endpoints(self):
-        self._add_discovery_uri_call()
         endpoints_data = [self._get_endpoint_v3_data(region='region1')
                           for e in range(0, 2)]
         endpoints_data.extend([self._get_endpoint_v3_data()
@@ -378,7 +380,6 @@ class TestCloudEndpoints(base.RequestsMockTestCase):
         self.assert_calls()
 
     def test_delete_endpoint(self):
-        self._add_discovery_uri_call()
         endpoint_data = self._get_endpoint_v3_data()
         self.register_uris([
             dict(method='GET',
