@@ -12,13 +12,15 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+# TODO(mordred) There are mocks of the image_client in here that are not
+#               using requests_mock. Erradicate them.
+
 import operator
 import tempfile
 import uuid
 
 import mock
 import munch
-import os_client_config as occ
 import six
 
 import shade
@@ -136,6 +138,74 @@ class TestImage(BaseTestImage):
             self.cloud.list_images())
         self.assert_calls()
 
+    def test_list_images_show_all(self):
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?member_status=all',
+                 json=self.fake_search_return)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([self.fake_image_dict]),
+            self.cloud.list_images(show_all=True))
+        self.assert_calls()
+
+    def test_list_images_show_all_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images?member_status=all',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([
+                self.fake_image_dict, deleted_image]),
+            self.cloud.list_images(show_all=True))
+        self.assert_calls()
+
+    def test_list_images_no_filter_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([
+                self.fake_image_dict, deleted_image]),
+            self.cloud.list_images(filter_deleted=False))
+        self.assert_calls()
+
+    def test_list_images_filter_deleted(self):
+        deleted_image = self.fake_image_dict.copy()
+        deleted_image['status'] = 'deleted'
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images',
+                 json={'images': [self.fake_image_dict, deleted_image]})
+        ])
+        self.assertEqual(
+            self.cloud._normalize_images([self.fake_image_dict]),
+            self.cloud.list_images())
+        self.assert_calls()
+
+    def test_list_images_string_properties(self):
+        image_dict = self.fake_image_dict.copy()
+        image_dict['properties'] = 'list,of,properties'
+        self.register_uris([
+            dict(method='GET', uri='https://image.example.com/v2/images',
+                 json={'images': [image_dict]}),
+        ])
+        images = self.cloud.list_images()
+        self.assertEqual(
+            self.cloud._normalize_images([image_dict]),
+            images)
+        self.assertEqual(
+            images[0]['properties']['properties'],
+            'list,of,properties')
+        self.assert_calls()
+
     def test_list_images_paginated(self):
         marker = str(uuid.uuid4())
         self.register_uris([
@@ -314,10 +384,12 @@ class TestImage(BaseTestImage):
             name, imagefile.name, wait=True, timeout=1,
             is_public=False, **kwargs)
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
-    def test_create_image_put_v1(self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '1'
+    def test_create_image_put_v1(
+            self, mock_image_client, mock_is_client_version):
+        # TODO(mordred) Fix this to use requests_mock
+        mock_is_client_version.return_value = False
         mock_image_client.get.return_value = []
         self.assertEqual([], self.cloud.list_images())
 
@@ -346,15 +418,15 @@ class TestImage(BaseTestImage):
                 'x-image-meta-checksum': mock.ANY,
                 'x-glance-registry-purge-props': 'false'
             })
-        mock_image_client.get.assert_called_with('/images/detail')
+        mock_image_client.get.assert_called_with('/images/detail', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_v1_bad_delete(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '1'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = False
         mock_image_client.get.return_value = []
         self.assertEqual([], self.cloud.list_images())
 
@@ -388,10 +460,11 @@ class TestImage(BaseTestImage):
             })
         mock_image_client.delete.assert_called_with('/images/42')
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
-    def test_update_image_no_patch(self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+    def test_update_image_no_patch(
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -415,14 +488,14 @@ class TestImage(BaseTestImage):
         self.cloud.update_image_properties(
             image=self._image_dict(ret),
             **{'owner_specified.shade.object': 'images/42 name'})
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         mock_image_client.patch.assert_not_called()
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_v2_bad_delete(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -457,11 +530,11 @@ class TestImage(BaseTestImage):
             data=mock.ANY)
         mock_image_client.delete.assert_called_with('/images/42')
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_bad_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         self.assertRaises(
@@ -469,11 +542,11 @@ class TestImage(BaseTestImage):
             self._call_create_image, '42 name', min_disk='fish', min_ram=0)
         mock_image_client.post.assert_not_called()
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_user_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         args = {'name': '42 name',
@@ -500,15 +573,15 @@ class TestImage(BaseTestImage):
             '/images/42/file',
             headers={'Content-Type': 'application/octet-stream'},
             data=mock.ANY)
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_meta_int(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -529,15 +602,15 @@ class TestImage(BaseTestImage):
         ret['status'] = 'success'
         mock_image_client.get.return_value = [ret]
         mock_image_client.post.return_value = ret
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_protected(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -571,11 +644,11 @@ class TestImage(BaseTestImage):
             headers={'Content-Type': 'application/octet-stream'})
         self.assertEqual(self._munch_images(ret), self.cloud.list_images())
 
-    @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
+    @mock.patch.object(shade.OpenStackCloud, '_is_client_version')
     @mock.patch.object(shade.OpenStackCloud, '_image_client')
     def test_create_image_put_user_prop(
-            self, mock_image_client, mock_api_version):
-        mock_api_version.return_value = '2'
+            self, mock_image_client, mock_is_client_version):
+        mock_is_client_version.return_value = True
         self.cloud.image_api_use_tasks = False
 
         mock_image_client.get.return_value = []
@@ -597,10 +670,22 @@ class TestImage(BaseTestImage):
         mock_image_client.post.return_value = ret
         self._call_create_image(
             '42 name', min_disk='0', min_ram=0, properties={'int_v': 12345})
-        mock_image_client.get.assert_called_with('/images')
+        mock_image_client.get.assert_called_with('/images', params={})
         self.assertEqual(
             self._munch_images(ret), self.cloud.list_images())
 
+    def test_get_image_by_id(self):
+        self.register_uris([
+            dict(method='GET',
+                 uri='https://image.example.com/v2/images/{id}'.format(
+                     id=self.image_id),
+                 json=self.fake_image_dict)
+        ])
+        self.assertEqual(
+            self.cloud._normalize_image(self.fake_image_dict),
+            self.cloud.get_image_by_id(self.image_id))
+        self.assert_calls()
+
 
 class TestImageV1Only(base.RequestsMockTestCase):
 
@@ -616,8 +701,7 @@ class TestImageV1Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v1/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '1', self.cloud_config.get_api_version('image'))
+        self.assertTrue(self.cloud._is_client_version('image', 1))
 
     def test_config_v2(self):
         self.cloud.cloud_config.config['image_api_version'] = '2'
@@ -626,8 +710,7 @@ class TestImageV1Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v1/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '1', self.cloud_config.get_api_version('image'))
+        self.assertFalse(self.cloud._is_client_version('image', 2))
 
 
 class TestImageV2Only(base.RequestsMockTestCase):
@@ -643,8 +726,7 @@ class TestImageV2Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v2/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '2', self.cloud_config.get_api_version('image'))
+        self.assertTrue(self.cloud._is_client_version('image', 2))
 
     def test_config_v2(self):
         self.cloud.cloud_config.config['image_api_version'] = '2'
@@ -653,26 +735,7 @@ class TestImageV2Only(base.RequestsMockTestCase):
         self.assertEqual(
             'https://image.example.com/v2/',
             self.cloud._image_client.get_endpoint())
-        self.assertEqual(
-            '2', self.cloud_config.get_api_version('image'))
-
-
-class TestImageVersionDiscovery(BaseTestImage):
-
-    def test_version_discovery_skip(self):
-        self.cloud.cloud_config.config['image_endpoint_override'] = \
-            'https://image.example.com/v2/override'
-
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/override/images',
-                 json={'images': []})
-        ])
-        self.assertEqual([], self.cloud.list_images())
-        self.assertEqual(
-            self.cloud._image_client.endpoint_override,
-            'https://image.example.com/v2/override')
-        self.assert_calls()
+        self.assertTrue(self.cloud._is_client_version('image', 2))
 
 
 class TestImageVolume(BaseTestImage):
@@ -758,24 +821,6 @@ class TestImageBrokenDiscovery(base.RequestsMockTestCase):
         ])
         self.assertEqual([], self.cloud.list_images())
         self.assertEqual(
-            self.cloud._image_client.endpoint_override,
+            self.cloud._image_client.get_endpoint(),
             'https://image.example.com/v2/')
         self.assert_calls()
-
-
-class TestImageDiscoveryOptimization(base.RequestsMockTestCase):
-
-    def setUp(self):
-        super(TestImageDiscoveryOptimization, self).setUp()
-        self.use_keystone_v3(catalog='catalog-versioned-image.json')
-
-    def test_version_discovery_skip(self):
-        self.cloud.cloud_config.config['image_api_version'] = '2'
-
-        self.register_uris([
-            dict(method='GET',
-                 uri='https://image.example.com/v2/images',
-                 json={'images': []})
-        ])
-        self.assertEqual([], self.cloud.list_images())
-        self.assert_calls()
