@@ -87,7 +87,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         mock_client.node.list.return_value = [m1]
         machines = self.op_cloud.list_machines()
         self.assertTrue(mock_client.node.list.called)
-        self.assertEqual(meta.obj_to_dict(m1), machines[0])
+        self.assertEqual(meta.obj_to_munch(m1), machines[0])
 
     @mock.patch.object(shade.OperatorCloud, 'ironic_client')
     def test_validate_node(self, mock_client):
@@ -102,7 +102,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         port1 = fakes.FakeMachinePort(1, "aa:bb:cc:dd", "node1")
         port2 = fakes.FakeMachinePort(2, "dd:cc:bb:aa", "node2")
         port_list = [port1, port2]
-        port_dict_list = meta.obj_list_to_dict(port_list)
+        port_dict_list = meta.obj_list_to_munch(port_list)
 
         mock_client.port.list.return_value = port_list
         nics = self.op_cloud.list_nics()
@@ -815,7 +815,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         mock_client.node.set_power_state.return_value = None
         node_id = 'node01'
         return_value = self.op_cloud.set_machine_power_on(node_id)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_power_state.assert_called_with(
             node_id='node01',
             state='on')
@@ -825,7 +825,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         mock_client.node.set_power_state.return_value = None
         node_id = 'node01'
         return_value = self.op_cloud.set_machine_power_off(node_id)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_power_state.assert_called_with(
             node_id='node01',
             state='off')
@@ -835,7 +835,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         mock_client.node.set_power_state.return_value = None
         node_id = 'node01'
         return_value = self.op_cloud.set_machine_power_reboot(node_id)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_power_state.assert_called_with(
             node_id='node01',
             state='reboot')
@@ -1008,7 +1008,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         return_value = self.op_cloud.activate_node(
             node_id,
             configdrive='http://127.0.0.1/file.iso')
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_provision_state.assert_called_with(
             node_uuid='node02',
             state='active',
@@ -1035,7 +1035,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
             configdrive='http://127.0.0.1/file.iso',
             wait=True,
             timeout=2)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_provision_state.assert_called_with(
             node_uuid='node04',
             state='active',
@@ -1049,7 +1049,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         node_id = 'node03'
         return_value = self.op_cloud.deactivate_node(
             node_id, wait=False)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_provision_state.assert_called_with(
             node_uuid='node03',
             state='deleted',
@@ -1073,7 +1073,7 @@ class TestShadeOperator(base.RequestsMockTestCase):
         node_id = 'node03'
         return_value = self.op_cloud.deactivate_node(
             node_id, wait=True, timeout=2)
-        self.assertEqual(None, return_value)
+        self.assertIsNone(return_value)
         mock_client.node.set_provision_state.assert_called_with(
             node_uuid='node03',
             state='deleted',
@@ -1192,17 +1192,23 @@ class TestShadeOperator(base.RequestsMockTestCase):
         get_session_mock.return_value = session_mock
         self.assertTrue(self.op_cloud.has_service("image"))
 
-    @mock.patch.object(shade.OpenStackCloud, 'nova_client')
-    def test_list_hypervisors(self, mock_nova):
+    def test_list_hypervisors(self):
         '''This test verifies that calling list_hypervisors results in a call
         to nova client.'''
-        mock_nova.hypervisors.list.return_value = [
-            fakes.FakeHypervisor('1', 'testserver1'),
-            fakes.FakeHypervisor('2', 'testserver2'),
-        ]
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'compute', 'public', append=['os-hypervisors', 'detail']),
+                 json={'hypervisors': [
+                     fakes.make_fake_hypervisor('1', 'testserver1'),
+                     fakes.make_fake_hypervisor('2', 'testserver2'),
+                 ]}),
+        ])
 
         r = self.op_cloud.list_hypervisors()
-        mock_nova.hypervisors.list.assert_called_once_with()
+
         self.assertEqual(2, len(r))
         self.assertEqual('testserver1', r[0]['hypervisor_hostname'])
         self.assertEqual('testserver2', r[1]['hypervisor_hostname'])
+
+        self.assert_calls()
