@@ -51,11 +51,8 @@ class TestUsers(base.RequestsMockTestCase):
                  uri=self._get_keystone_mock_url(resource='users', v3=False),
                  status_code=200,
                  json=user_data.json_response,
-                 validate=user_data.json_request),
-            dict(method='GET',
-                 uri=self._get_keystone_mock_url(resource='users', v3=False,
-                                                 append=[user_data.user_id]),
-                 status_code=200, json=user_data.json_response)])
+                 validate=dict(json=user_data.json_request)),
+        ])
 
         user = self.op_cloud.create_user(
             name=user_data.name, email=user_data.email,
@@ -75,11 +72,8 @@ class TestUsers(base.RequestsMockTestCase):
             dict(method='POST',
                  uri=self._get_keystone_mock_url(resource='users'),
                  status_code=200, json=user_data.json_response,
-                 validate=user_data.json_request),
-            dict(method='GET',
-                 uri=self._get_keystone_mock_url(
-                     resource='users', append=[user_data.user_id]),
-                 status_code=200, json=user_data.json_response)])
+                 validate=dict(json=user_data.json_request)),
+        ])
 
         user = self.op_cloud.create_user(
             name=user_data.name, email=user_data.email,
@@ -125,9 +119,7 @@ class TestUsers(base.RequestsMockTestCase):
                  json=user_data.json_response),
             dict(method='PUT', uri=mock_user_resource_uri, status_code=200,
                  json=user_data.json_response,
-                 validate=dict(json={'user': {}})),
-            dict(method='GET', uri=mock_user_resource_uri, status_code=200,
-                 json=user_data.json_response)])
+                 validate=dict(json={'user': {}}))])
 
         user = self.op_cloud.update_user(
             user_data.user_id, password=user_data.password)
