@@ -25,7 +25,7 @@ class TestVolume(base.RequestsMockTestCase):
         server = dict(id='server001')
         vol = {'id': 'volume001', 'status': 'available',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         rattach = {'server_id': server['id'], 'device': 'device001',
                    'volumeId': volume['id'], 'id': 'attachmentId'}
         self.register_uris([
@@ -47,7 +47,7 @@ class TestVolume(base.RequestsMockTestCase):
         server = dict(id='server001')
         vol = {'id': 'volume001', 'status': 'available',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='POST',
                  uri=self.get_mock_url(
@@ -71,11 +71,11 @@ class TestVolume(base.RequestsMockTestCase):
         server = dict(id='server001')
         vol = {'id': 'volume001', 'status': 'available',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         vol['attachments'] = [{'server_id': server['id'],
                                'device': 'device001'}]
         vol['status'] = 'attached'
-        attached_volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        attached_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         rattach = {'server_id': server['id'], 'device': 'device001',
                    'volumeId': volume['id'], 'id': 'attachmentId'}
         self.register_uris([
@@ -105,9 +105,9 @@ class TestVolume(base.RequestsMockTestCase):
         server = dict(id='server001')
         vol = {'id': 'volume001', 'status': 'available',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         vol['status'] = 'error'
-        errored_volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        errored_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         rattach = {'server_id': server['id'], 'device': 'device001',
                    'volumeId': volume['id'], 'id': 'attachmentId'}
         self.register_uris([
@@ -201,10 +201,10 @@ class TestVolume(base.RequestsMockTestCase):
         attachments = [{'server_id': 'server001', 'device': 'device001'}]
         vol = {'id': 'volume001', 'status': 'attached', 'name': '',
                'attachments': attachments}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         vol['status'] = 'available'
         vol['attachments'] = []
-        avail_volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        avail_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='DELETE',
                  uri=self.get_mock_url(
@@ -223,10 +223,10 @@ class TestVolume(base.RequestsMockTestCase):
         attachments = [{'server_id': 'server001', 'device': 'device001'}]
         vol = {'id': 'volume001', 'status': 'attached', 'name': '',
                'attachments': attachments}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         vol['status'] = 'error'
         vol['attachments'] = []
-        errored_volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        errored_volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='DELETE',
                  uri=self.get_mock_url(
@@ -247,7 +247,7 @@ class TestVolume(base.RequestsMockTestCase):
     def test_delete_volume_deletes(self):
         vol = {'id': 'volume001', 'status': 'attached',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
@@ -266,7 +266,7 @@ class TestVolume(base.RequestsMockTestCase):
     def test_delete_volume_gone_away(self):
         vol = {'id': 'volume001', 'status': 'attached',
                'name': '', 'attachments': []}
-        volume = meta.obj_to_dict(fakes.FakeVolume(**vol))
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
@@ -279,20 +279,172 @@ class TestVolume(base.RequestsMockTestCase):
         self.assertFalse(self.cloud.delete_volume(volume['id']))
         self.assert_calls()
 
-    def test_list_volumes_with_pagination(self):
-        vol1 = meta.obj_to_dict(fakes.FakeVolume('01', 'available', 'vol1'))
-        vol2 = meta.obj_to_dict(fakes.FakeVolume('02', 'available', 'vol2'))
+    def test_delete_volume_force(self):
+        vol = {'id': 'volume001', 'status': 'attached',
+               'name': '', 'attachments': []}
+        volume = meta.obj_to_munch(fakes.FakeVolume(**vol))
         self.register_uris([
             dict(method='GET',
                  uri=self.get_mock_url(
                      'volumev2', 'public', append=['volumes', 'detail']),
+                 json={'volumes': [volume]}),
+            dict(method='POST',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', volume.id, 'action']),
+                 json={'os-force_delete': None}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public', append=['volumes', 'detail']),
+                 json={'volumes': []})])
+        self.assertTrue(self.cloud.delete_volume(volume['id'], force=True))
+        self.assert_calls()
+
+    def test_list_volumes_with_pagination(self):
+        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
+        vol2 = meta.obj_to_munch(fakes.FakeVolume('02', 'available', 'vol2'))
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail']),
                  json={
-                     'volumes': [vol1, vol2],
+                     'volumes': [vol1],
                      'volumes_links': [
-                         {'href': 'https://volume.example.com/fake_url',
-                          'rel': 'next'}]})])
+                         {'href': self.get_mock_url(
+                             'volumev2', 'public',
+                             append=['volumes', 'detail'],
+                             qs_elements=['marker=01']),
+                          'rel': 'next'}]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail'],
+                     qs_elements=['marker=01']),
+                 json={
+                     'volumes': [vol2],
+                     'volumes_links': [
+                         {'href': self.get_mock_url(
+                             'volumev2', 'public',
+                             append=['volumes', 'detail'],
+                             qs_elements=['marker=02']),
+                          'rel': 'next'}]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail'],
+                     qs_elements=['marker=02']),
+                 json={'volumes': []})])
         self.assertEqual(
             [self.cloud._normalize_volume(vol1),
              self.cloud._normalize_volume(vol2)],
             self.cloud.list_volumes())
         self.assert_calls()
+
+    def test_list_volumes_with_pagination_next_link_fails_once(self):
+        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
+        vol2 = meta.obj_to_munch(fakes.FakeVolume('02', 'available', 'vol2'))
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail']),
+                 json={
+                     'volumes': [vol1],
+                     'volumes_links': [
+                         {'href': self.get_mock_url(
+                             'volumev2', 'public',
+                             append=['volumes', 'detail'],
+                             qs_elements=['marker=01']),
+                          'rel': 'next'}]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail'],
+                     qs_elements=['marker=01']),
+                 status_code=404),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail']),
+                 json={
+                     'volumes': [vol1],
+                     'volumes_links': [
+                         {'href': self.get_mock_url(
+                             'volumev2', 'public',
+                             append=['volumes', 'detail'],
+                             qs_elements=['marker=01']),
+                          'rel': 'next'}]}),
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail'],
+                     qs_elements=['marker=01']),
+                 json={
+                     'volumes': [vol2],
+                     'volumes_links': [
+                         {'href': self.get_mock_url(
+                             'volumev2', 'public',
+                             append=['volumes', 'detail'],
+                             qs_elements=['marker=02']),
+                          'rel': 'next'}]}),
+
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', 'detail'],
+                     qs_elements=['marker=02']),
+                 json={'volumes': []})])
+        self.assertEqual(
+            [self.cloud._normalize_volume(vol1),
+             self.cloud._normalize_volume(vol2)],
+            self.cloud.list_volumes())
+        self.assert_calls()
+
+    def test_list_volumes_with_pagination_next_link_fails_all_attempts(self):
+        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
+        uris = []
+        attempts = 5
+        for i in range(attempts):
+            uris.extend([
+                dict(method='GET',
+                     uri=self.get_mock_url(
+                         'volumev2', 'public',
+                         append=['volumes', 'detail']),
+                     json={
+                         'volumes': [vol1],
+                         'volumes_links': [
+                             {'href': self.get_mock_url(
+                                 'volumev2', 'public',
+                                 append=['volumes', 'detail'],
+                                 qs_elements=['marker=01']),
+                              'rel': 'next'}]}),
+                dict(method='GET',
+                     uri=self.get_mock_url(
+                         'volumev2', 'public',
+                         append=['volumes', 'detail'],
+                         qs_elements=['marker=01']),
+                     status_code=404)])
+        self.register_uris(uris)
+        # Check that found volumes are returned even if pagination didn't
+        # complete because call to get next link 404'ed for all the allowed
+        # attempts
+        self.assertEqual(
+            [self.cloud._normalize_volume(vol1)],
+            self.cloud.list_volumes())
+        self.assert_calls()
+
+    def test_get_volume_by_id(self):
+        vol1 = meta.obj_to_munch(fakes.FakeVolume('01', 'available', 'vol1'))
+        self.register_uris([
+            dict(method='GET',
+                 uri=self.get_mock_url(
+                     'volumev2', 'public',
+                     append=['volumes', '01']),
+                 json={'volume': vol1}
+                 )
+        ])
+        self.assertEqual(
+            self.cloud._normalize_volume(vol1),
+            self.cloud.get_volume_by_id('01'))
+        self.assert_calls()
